@@ -3,7 +3,6 @@ import datetime as dt
 from abc import ABC, abstractmethod
 from copy import deepcopy
 from dataclasses import replace
-from enum import Enum
 from pathlib import Path
 from textwrap import dedent
 from typing import Any, Dict, List, Optional, Tuple, Union, cast
@@ -15,6 +14,7 @@ import xarray as xr
 from numpy.distutils.misc_util import is_sequence
 
 from ravenpy import __version__
+from ravenpy.config import options
 from ravenpy.config.commands import (
     HRU,
     BaseDataCommand,
@@ -30,6 +30,7 @@ from ravenpy.config.commands import (
     HRUStateVariableTableCommand,
     LandUseClassesCommand,
     ObservationDataCommand,
+    RedirectToFileCommand,
     ReservoirCommand,
     SBGroupPropertyMultiplierCommand,
     SoilClassesCommand,
@@ -258,10 +259,9 @@ class RVI(RV):
     :EvaluationMetrics     {evaluation_metrics}
     {evaluation_periods}
     :WriteNetcdfFormat     yes
-    #:WriteForcingFunctions
     :SilentMode
     :PavicsMode
-    {suppress_output}
+    {suppress_output}{write_forcing_functions}{custom_output}
 
     :NetCDFAttribute title Simulated river discharge
     :NetCDFAttribute history Created on {now} by Raven
@@ -272,61 +272,13 @@ class RVI(RV):
     :NetCDFAttribute time_coverage_start {start_date}
     :NetCDFAttribute time_coverage_end {end_date}
     """
-
-    class EvaporationOptions(Enum):
-        PET_CONSTANT = "PET_CONSTANT"
-        PET_PENMAN_MONTEITH = "PET_PENMAN_MONTEITH"
-        PET_PENMAN_COMBINATION = "PET_PENMAN_COMBINATION"
-        PET_PRIESTLEY_TAYLOR = "PET_PRIESTLEY_TAYLOR"
-        PET_HARGREAVES = "PET_HARGREAVES"
-        PET_HARGREAVES_1985 = "PET_HARGREAVES_1985"
-        PET_FROMMONTHLY = "PET_FROMMONTHLY"
-        PET_DATA = "PET_DATA"
-        PET_HAMON_1961 = "PET_HAMON_1961"
-        PET_TURC_1961 = "PET_TURC_1961"
-        PET_MAKKINK_1957 = "PET_MAKKINK_1957"
-        PET_MONTHLY_FACTOR = "PET_MONTHLY_FACTOR"
-        PET_MOHYSE = "PET_MOHYSE"
-        PET_OUDIN = "PET_OUDIN"
-
-    class CalendarOptions(Enum):
-        PROLEPTIC_GREGORIAN = "PROLEPTIC_GREGORIAN"
-        JULIAN = "JULIAN"
-        GREGORIAN = "GREGORIAN"
-        STANDARD = "STANDARD"
-        NOLEAP = "NOLEAP"
-        _365_DAY = "365_DAY"
-        ALL_LEAP = "ALL_LEAP"
-        _366_DAY = "366_DAY"
-
-    class EvaluationMetrics(Enum):
-        NASH_SUTCLIFFE = "NASH_SUTCLIFFE"
-        LOG_NASH = "LOG_NASH"
-        RMSE = "RMSE"
-        PCT_BIAS = "PCT_BIAS"
-        ABSERR = "ABSERR"
-        ABSMAX = "ABSMAX"
-        PDIFF = "PDIFF"
-        TMVOL = "TMVOL"
-        RCOEFF = "RCOEFF"
-        NSC = "NSC"
-        KLING_GUPTA = "KLING_GUPTA"
-
-    class RainSnowFractionOptions(Enum):
-        DATA = "RAINSNOW_DATA"
-        DINGMAN = "RAINSNOW_DINGMAN"
-        UBC = "RAINSNOW_UBC"
-        HBV = "RAINSNOW_HBV"
-        HARDER = "RAINSNOW_HARDER"
-        HSPF = "RAINSNOW_HSPF"
-
-    class RoutingOptions(Enum):
-        DIFFUSIVE_WAVE = "ROUTE_DIFFUSIVE_WAVE"
-        HYDROLOGIC = "ROUTE_HYDROLOGIC"
-        NONE = "ROUTE_NONE"
-        STORAGE_COEFF = "ROUTE_STORAGE_COEFF"
-        PLUG_FLOW = "ROUTE_PLUG_FLOW"
-        MUSKINGUM = "MUSKINGUM"
+    # For backward compatibility
+    EvaporationOptions = options.Evaporation
+    CalendarOptions = options.Calendar
+    EvaluationMetrics = options.EvaluationMetrics
+    RainSnowFractionOptions = options.RainSnowFraction
+    RoutingOptions = options.Routing
+    #
 
     def __init__(self, config):
         super().__init__(config)
@@ -339,22 +291,25 @@ class RVI(RV):
         # These correspond to properties whose setters will pass their value through
         # an Enum cast (triggering a ValueError at runtime for unknown values) and
         # getters will be used when rendering the template in the `to_rv` method
-        self._calendar = RVI.CalendarOptions.STANDARD
-        self._routing = RVI.RoutingOptions.NONE
+        self._calendar = options.Calendar.STANDARD
+        self._routing = options.Routing.NONE
         self._start_date = None
         self._end_date = None
-        self._rain_snow_fraction = RVI.RainSnowFractionOptions.DATA
+        self._rain_snow_fraction = options.RainSnowFraction.DATA
         self._evaporation = None
         self._ow_evaporation = None
         self._duration = 1
         self._evaluation_metrics = [
-            RVI.EvaluationMetrics.NASH_SUTCLIFFE,
-            RVI.EvaluationMetrics.RMSE,
+            options.EvaluationMetrics.NASH_SUTCLIFFE,
+            options.EvaluationMetrics.RMSE,
         ]
         self._evaluation_periods = []
         self._suppress_output = False
+        self._write_forcing_functions = False
+        self._custom_output = []
 
     def configure_from_nc_data(self, fns):
+
         with xr.open_mfdataset(fns, combine="by_coords") as ds:
             start, end = ds.indexes["time"][0], ds.indexes["time"][-1]
             cal = ds.time.encoding.get("calendar", "standard")
@@ -365,7 +320,7 @@ class RVI(RV):
         if self.end_date in [None, dt.datetime(1, 1, 1)]:
             self.end_date = end
 
-        self.calendar = RVI.CalendarOptions(cal.upper())
+        self.calendar = options.Calendar(cal.upper())
 
     @property
     def raven_version(self):
@@ -433,7 +388,7 @@ class RVI(RV):
         ms = []
         for v in values:
             v = v.upper() if isinstance(v, str) else v.value
-            ms.append(RVI.EvaluationMetrics(v))
+            ms.append(options.EvaluationMetrics(v))
         self._evaluation_metrics = ms
 
     @property
@@ -447,6 +402,16 @@ class RVI(RV):
             values = [values]
         self._evaluation_periods = values
 
+    @property
+    def custom_output(self):
+        return "\n".join([str(o) for o in self._custom_output])
+
+    @custom_output.setter
+    def custom_output(self, values):
+        if not isinstance(values, (list, set, tuple)):
+            values = [values]
+        self._custom_output = values
+
     def _update_duration(self):
         if self.end_date is not None and self.start_date is not None:
             self._duration = (self.end_date - self.start_date).days
@@ -457,7 +422,7 @@ class RVI(RV):
 
     @property
     def suppress_output(self):
-        tag = ":SuppressOutput\n:DontWriteWatershedStorage"
+        tag = ":SuppressOutput\n:DontWriteWatershedStorage\n"
         return tag if self._suppress_output else ""
 
     @suppress_output.setter
@@ -466,6 +431,17 @@ class RVI(RV):
             raise ValueError
         self._suppress_output = value
 
+    @property
+    def write_forcing_functions(self):
+        tag = ":WriteForcingFunctions\n"
+        return tag if self._write_forcing_functions else ""
+
+    @write_forcing_functions.setter
+    def write_forcing_functions(self, value):
+        if not isinstance(value, bool):
+            raise ValueError
+        self._write_forcing_functions = value
+
     @property
     def routing(self):
         return self._routing.value
@@ -473,7 +449,7 @@ class RVI(RV):
     @routing.setter
     def routing(self, value):
         v = value.upper() if isinstance(value, str) else value.value
-        self._routing = RVI.RoutingOptions(v)
+        self._routing = options.Routing(v)
 
     @property
     def rain_snow_fraction(self):
@@ -483,7 +459,7 @@ class RVI(RV):
     @rain_snow_fraction.setter
     def rain_snow_fraction(self, value):
         v = value.upper() if isinstance(value, str) else value.value
-        self._rain_snow_fraction = RVI.RainSnowFractionOptions(v)
+        self._rain_snow_fraction = options.RainSnowFraction(v)
 
     @property
     def evaporation(self):
@@ -493,7 +469,7 @@ class RVI(RV):
     @evaporation.setter
     def evaporation(self, value):
         v = value.upper() if isinstance(value, str) else value.value
-        self._evaporation = RVI.EvaporationOptions(v)
+        self._evaporation = options.Evaporation(v)
 
     @property
     def ow_evaporation(self):
@@ -503,7 +479,7 @@ class RVI(RV):
     @ow_evaporation.setter
     def ow_evaporation(self, value):
         v = value.upper() if isinstance(value, str) else value.value
-        self._ow_evaporation = RVI.EvaporationOptions(v)
+        self._ow_evaporation = options.Evaporation(v)
 
     @property
     def calendar(self):
@@ -513,7 +489,7 @@ class RVI(RV):
     @calendar.setter
     def calendar(self, value):
         v = value.upper() if isinstance(value, str) else value.value
-        self._calendar = RVI.CalendarOptions(v)
+        self._calendar = options.Calendar(v)
 
     def _dt2cf(self, date):
         """Convert datetime to cftime datetime."""
@@ -656,8 +632,14 @@ class RVT(RV):
         # Specifies whether the variables must be configured using file NC data
         self._auto_nc_configure = True
 
-        self.nc_index = 0
-        self.grid_weights = None
+        self.nc_index = self.meteo_idx = 0  # For meteo forcing files
+
+        self.hydro_idx: Tuple = (0,)  # Streamflow observation file station index
+        self.gauged_sb_ids: Tuple = (
+            None,
+        )  # Corresponding sub-basin ID. If (None,), will try to guess.
+
+        self.grid_weights: Union[GridWeightsCommand, RedirectToFileCommand, None] = None
         self.rain_correction = None
         self.snow_correction = None
         self.monthly_ave_evaporation = None
@@ -695,7 +677,10 @@ class RVT(RV):
             else:
                 cmd = DataCommand(**kwargs)
         else:
-            cmd = GriddedForcingCommand(**kwargs)
+            if std_name == "water_volume_transport_in_river_channel" or is_obs_var:
+                cmd = ObservationDataCommand(**kwargs)
+            else:
+                cmd = GriddedForcingCommand(**kwargs)
 
         spec = self._var_specs[std_name]
         self._var_cmds[std_name] = replace(cmd, **spec)
@@ -710,20 +695,35 @@ class RVT(RV):
         self._auto_nc_configure = False
 
     def configure_from_nc_data(self, fns):
+        from ravenpy.utilities.coords import infer_scale_and_offset
 
         assert self._auto_nc_configure is True
 
         self._var_cmds = {k: None for k in RVT.NC_VARS.keys()}
 
         for fn in fns:
+            if isinstance(fn, str) and not fn.startswith("http"):
+                fn = Path(fn)
             with xr.open_dataset(fn) as ds:
                 try:
                     self._nc_latitude = ds.cf["latitude"]
                     self._nc_longitude = ds.cf["longitude"]
-                    self._nc_elevation = ds.cf["vertical"]
+                    latitude_var_name_nc = self._nc_latitude.name
+                    longitude_var_name_nc = self._nc_longitude.name
                 except KeyError:
                     # Will try to compute values later from first HRU (in self.to_rv)
-                    pass
+                    latitude_var_name_nc = ""
+                    longitude_var_name_nc = ""
+
+                try:
+                    self._nc_elevation = ds.cf["vertical"]
+                    elevation_var_name_nc = self._nc_elevation.name
+                except KeyError:
+                    if "elevation" in ds:
+                        self._nc_elevation = ds["elevation"]
+                        elevation_var_name_nc = "elevation"
+                    else:
+                        elevation_var_name_nc = ""
 
                 if "station_id" in ds:
                     self._station_id = ds["station_id"]
@@ -734,14 +734,25 @@ class RVT(RV):
                         if var_name not in ds.data_vars:
                             continue
                         nc_var = ds[var_name]
-                        self._add_nc_variable(
+                        data_type = RVT.NC_VARS[std_name]["raven"]
+                        specs = dict(
                             name=std_name,
                             file_name_nc=fn,
-                            data_type=RVT.NC_VARS[std_name]["raven"],
+                            data_type=data_type,
                             var_name_nc=var_name,
+                            latitude_var_name_nc=latitude_var_name_nc,
+                            longitude_var_name_nc=longitude_var_name_nc,
+                            elevation_var_name_nc=elevation_var_name_nc,
                             dim_names_nc=nc_var.dims,
                             units=nc_var.attrs.get("units"),
                         )
+                        # Infer scale and offset parameters for unit conversion.
+                        if specs["units"] is not None:
+                            specs["scale"], specs["offset"] = infer_scale_and_offset(
+                                nc_var, data_type
+                            )
+
+                        self._add_nc_variable(**specs)
                         self._number_grid_cells = int(nc_var.size / len(ds["time"]))
                         break
 
@@ -749,6 +760,9 @@ class RVT(RV):
         if key in self._var_specs:
             self._var_specs[key].update(value)
             return True
+        # For backward compatibility
+        if key == "nc_index":
+            key = "meteo_idx"
         return super().update(key, value)
 
     def to_rv(self):
@@ -767,7 +781,7 @@ class RVT(RV):
         use_gauge = any(type(cmd) is DataCommand for cmd in self._var_cmds.values())
         if use_gauge:
             gauges = []
-            for idx in np.atleast_1d(self.nc_index):
+            for idx in np.atleast_1d(self.meteo_idx):
                 data_cmds = []
                 for var, cmd in self._var_cmds.items():
                     if cmd and not isinstance(cmd, ObservationDataCommand):
@@ -807,7 +821,7 @@ class RVT(RV):
                     and self._station_id.shape
                     and len(self._station_id) > idx
                 ):
-                    name = self._station_id[idx]
+                    name = str(self._station_id.values[idx])
                 else:
                     name = f"default_{idx + 1}"
 
@@ -827,7 +841,7 @@ class RVT(RV):
             d["gauge"] = "\n".join([str(g) for g in gauges])
         else:
             # Construct default grid weights applying equally to all HRUs
-            data = [(hru.hru_id, self.nc_index, 1.0) for hru in self._config.rvh.hrus]
+            data = [(hru.hru_id, self.meteo_idx, 1.0) for hru in self._config.rvh.hrus]
             gw = self.grid_weights or GridWeightsCommand(
                 number_hrus=len(data),
                 number_grid_cells=self._number_grid_cells,
@@ -836,32 +850,38 @@ class RVT(RV):
             cmds = []
             for var, cmd in self._var_cmds.items():
                 if cmd and not isinstance(cmd, ObservationDataCommand):
-                    # TODO: implement a RedirectToFile mechanism to avoid inlining the grid weights
-                    # multiple times as we do here
                     cmd = cast(Union[GriddedForcingCommand, StationForcingCommand], cmd)
                     if len(cmd.grid_weights.data) == 1:
                         cmd.grid_weights = gw
                     cmds.append(cmd)
             d["forcing_list"] = "\n".join(map(str, cmds))
 
-        # QUESTION: is it possible to have (and if yes should we support) more than 1
-        # observation variable? For now we don't.
         for cmd in self._var_cmds.values():
+            observed_data = []
             if isinstance(cmd, ObservationDataCommand) and self._config is not None:
-                # Search for the gauged SB, not sure what should happen when there are
-                # more than one (should it be even supported?)
-                for sb in self._config.rvh.subbasins:
-                    if sb.gauged:
-                        cmd.subbasin_id = sb.subbasin_id
-                        break
-                else:
-                    raise Exception(
-                        "Could not find an outlet subbasin for observation data"
+                if self.gauged_sb_ids == (None,):
+                    # Search for single gauged sub-basin
+                    for sb in self._config.rvh.subbasins:
+                        if sb.gauged:
+                            self.gauged_sb_ids = (sb.subbasin_id,)
+                            break
+                    else:
+                        raise Exception(
+                            "Could not find an outlet subbasin for observation data"
+                        )
+
+                if len(self.gauged_sb_ids) != len(self.hydro_idx):
+                    raise ValueError(
+                        "`gauged_sb_ids` and `hydro_idx` must have the same number of entries."
                     )
-                # Set the :StationxIdx (which starts at 1)
-                cmd.index = self.nc_index + 1
-                d["observed_data"] = cmd  # type: ignore
-                break
+
+                for idx, sb_id in zip(self.hydro_idx, self.gauged_sb_ids):
+                    cmd = deepcopy(cast(ObservationDataCommand, cmd))
+                    cmd.index = idx + 1  # Python index to Raven index
+                    cmd.subbasin_id = sb_id
+                    observed_data.append(cmd)
+
+                d["observed_data"] = "\n".join(map(str, observed_data))  # type: ignore
 
         return super().to_rv(dedent(self.tmpl.lstrip("\n")).format(**d), "RVT")
 
