@@ -26,6 +26,7 @@ import numpy as np
 import xarray as xr
 from numpy.distutils.misc_util import is_sequence
 
+from ravenpy.config.commands import RedirectToFileCommand
 from ravenpy.config.rvs import RVC, Config
 
 RAVEN_EXEC_PATH = os.getenv("RAVENPY_RAVEN_BINARY_PATH") or shutil.which("raven")
@@ -227,6 +228,8 @@ class Raven:
         output/
 
         """
+        self._rv_paths = []
+
         if overwrite:
             if self.model_path.exists():
                 shutil.rmtree(str(self.exec_path))
@@ -257,10 +260,19 @@ class Raven:
 
         self._dump_rv()
 
-        # Create symbolic link to input files
+        # Create symbolic link to input files that are not DAP links
         for fn in ts:
-            if not (self.model_path / Path(fn).name).exists():
-                os.symlink(os.path.realpath(fn), str(self.model_path / Path(fn).name))
+            if isinstance(fn, Path) or not fn.startswith("http"):
+                if not (self.model_path / Path(fn).name).exists():
+                    os.symlink(
+                        os.path.realpath(fn), str(self.model_path / Path(fn).name)
+                    )
+
+        # Special case where a RedirectToFile command is used for the grid weights, which
+        # needs to be symlinked also
+        if isinstance(self.config.rvt.grid_weights, RedirectToFileCommand):
+            fn = self.config.rvt.grid_weights.path
+            os.symlink(os.path.realpath(fn), str(self.model_path / Path(fn).name))
 
         # Create symbolic link to Raven executable
         if not self.raven_cmd.exists():
@@ -318,7 +330,7 @@ class Raven:
             self._pdim = "params"
         elif "hru_state" in parallel.keys() or "basin_state" in parallel.keys():
             self._pdim = "state"
-        elif "nc_index" in parallel.keys():
+        elif "meteo_idx" in parallel.keys() or "nc_index" in parallel.keys():
             self._pdim = "nbasins"
         else:
             self._pdim = "pdim"
@@ -352,7 +364,7 @@ class Raven:
                     else:
                         self.config.update(key, val[self.psim])
 
-            cmd = self.setup_model_run(tuple(map(Path, ts)))
+            cmd = self.setup_model_run(ts)
 
             procs.append(
                 subprocess.Popen(
@@ -474,8 +486,8 @@ class Raven:
         # Try to create a zip file
         with zipfile.ZipFile(outfn, "w") as f:
             for fn in files:
-                len(fn.parts)
-                f.write(fn, arcname=fn.relative_to(Path(*fn.parts[:i])))
+                arcname = fn.relative_to(Path(*fn.parts[:i]))
+                f.write(fn, arcname=arcname)
 
         return outfn
 
@@ -836,6 +848,15 @@ class Ostrich(Raven):
 
     @property
     def obj_func(self):
+        """Value of objective function reached by calibration.
+
+        Note that the Ostrich optimizer seeks to minimise the objective function. To optimize diagnostic
+        metrics such as the Nash-Sutcliffe Efficiency, it needs to be multiplied by -1.
+
+        See Also
+        --------
+        See the `diagnostics` method to obtain the diagnostic metrics values.
+        """
         return np.loadtxt(self.outputs["params_seq"], skiprows=1)[-1, 1]
 
     @property
