@@ -4,6 +4,7 @@ from typing import cast
 
 from pydantic.dataclasses import dataclass
 
+from ravenpy.config import options
 from ravenpy.config.commands import HRU, LU, BasinIndexCommand, HRUState, Sub
 from ravenpy.config.rvs import RVI
 from ravenpy.models.base import Ostrich, Raven
@@ -179,7 +180,7 @@ class GR4JCN(Raven):
         """
         self.config.rvi.set_tmpl(rvi_tmpl)
 
-        self.config.rvi.rain_snow_fraction = RVI.RainSnowFractionOptions.DINGMAN
+        self.config.rvi.rain_snow_fraction = options.RainSnowFraction.DINGMAN
         self.config.rvi.evaporation = "PET_OUDIN"
 
     def derived_parameters(self):
@@ -203,7 +204,7 @@ class GR4JCN(Raven):
             for hru in self.config.rvh.hrus:
                 if isinstance(hru, GR4JCN.LandHRU) or hru.hru_type == "land":
                     self.config.rvc.hru_states[hru.hru_id] = HRUState(
-                        index=hru.hru_id, soil0=soil0, soil1=soil1
+                        index=hru.hru_id, data={"SOIL[0]": soil0, "SOIL[1]": soil1}
                     )
                 elif isinstance(hru, GR4JCN.LakeHRU) or hru.hru_type == "lake":
                     self.config.rvc.hru_states[hru.hru_id] = HRUState(index=hru.hru_id)
