@@ -6,6 +6,7 @@ from typing import cast
 import xarray as xr
 from pydantic.dataclasses import dataclass
 
+from ravenpy.config import options
 from ravenpy.config.commands import (
     HRU,
     LU,
@@ -14,7 +15,7 @@ from ravenpy.config.commands import (
     HRUState,
     Sub,
 )
-from ravenpy.config.rvs import RVH, RVI
+from ravenpy.config.rvs import RVH
 from ravenpy.models.base import Ostrich, Raven
 
 
@@ -241,10 +242,10 @@ class HYPR(Raven):
         """
         self.config.rvi.set_tmpl(rvi_tmpl)
 
-        self.config.rvi.routing = RVI.RoutingOptions.NONE
-        self.config.rvi.rain_snow_fraction = RVI.RainSnowFractionOptions.HBV
-        self.config.rvi.evaporation = RVI.EvaporationOptions.PET_FROMMONTHLY
-        self.config.rvi.ow_evaporation = RVI.EvaporationOptions.PET_FROMMONTHLY
+        self.config.rvi.routing = options.Routing.NONE
+        self.config.rvi.rain_snow_fraction = options.RainSnowFraction.HBV
+        self.config.rvi.evaporation = options.Evaporation.PET_FROMMONTHLY
+        self.config.rvi.ow_evaporation = options.Evaporation.PET_FROMMONTHLY
 
         #########
         # R V H #
@@ -266,7 +267,9 @@ class HYPR(Raven):
         # R V C #
         #########
 
-        self.config.rvc.hru_states[1] = HRUState(soil0=10, soil1=10, depression=20)
+        self.config.rvc.hru_states[1] = HRUState(
+            data={"SOIL[0]": 10, "SOIL[1]": 10, "DEPRESSION": 20}
+        )
 
     def derived_parameters(self):
         params = cast(HYPR.Params, self.config.rvp.params)
