@@ -4,8 +4,8 @@ from typing import cast
 
 from pydantic.dataclasses import dataclass
 
+from ravenpy.config import options
 from ravenpy.config.commands import HRU, LU, BasinIndexCommand, HRUState, Sub
-from ravenpy.config.rvs import RVI
 from ravenpy.models.base import Ostrich, Raven
 
 from .gr4jcn import GR4JCN
@@ -198,7 +198,7 @@ class MOHYSE(Raven):
         # R V I #
         #########
 
-        self.config.rvi.rain_snow_fraction = RVI.RainSnowFractionOptions.DATA
+        self.config.rvi.rain_snow_fraction = options.RainSnowFraction.DATA
         self.config.rvi.evaporation = "PET_MOHYSE"
 
     def derived_parameters(self):
