@@ -4,8 +4,8 @@ from typing import cast
 
 from pydantic.dataclasses import dataclass
 
+from ravenpy.config import options
 from ravenpy.config.commands import HRU, LU, BasinIndexCommand, HRUState, Sub
-from ravenpy.config.rvs import RVI
 from ravenpy.models.base import Ostrich, Raven
 
 
@@ -267,15 +267,17 @@ class SACSMA(Raven):
         """
         self.config.rvi.set_tmpl(rvi_tmpl)
 
-        self.config.rvi.rain_snow_fraction = RVI.RainSnowFractionOptions.DATA
-        self.config.rvi.evaporation = RVI.EvaporationOptions.PET_OUDIN
+        self.config.rvi.rain_snow_fraction = options.RainSnowFraction.DATA
+        self.config.rvi.evaporation = options.Evaporation.PET_OUDIN
 
     def derived_parameters(self):
         params = cast(SACSMA.Params, self.config.rvp.params)
 
         soil0 = params.par_x04 * 1000.0
         soil2 = params.par_x06 * 1000.0
-        self.config.rvc.hru_states[1] = HRUState(soil0=soil0, soil2=soil2)
+        self.config.rvc.hru_states[1] = HRUState(
+            data={"SOIL[0]": soil0, "SOIL[2]": soil2}
+        )
 
         self.config.rvt.rain_correction = params.par_x20
         self.config.rvt.snow_correction = params.par_x21
