@@ -12,7 +12,6 @@ import warnings
 from pathlib import Path
 from typing import List, Tuple
 
-# import climpred
 import numpy as np
 import pandas as pd
 import xarray as xr
@@ -101,6 +100,8 @@ def perform_climatology_esp(
       Array of streamflow values from the ESP method along with list of member years
 
     """
+    original_run_name = kwds.get("run_name", "esp")
+
     # Get the timeseries
     tsnc = xr.open_dataset(kwds["ts"])
 
@@ -150,6 +151,7 @@ def perform_climatology_esp(
         rvc = kwds.pop("rvc")
     else:
         # Run model to get rvc file after warm-up using base meteo
+        kwds["run_name"] = f"{original_run_name}_warmup"
         rvc = get_raven_states(model_name, workdir=workdir, **kwds)
 
     # We need to check which years are long enough (ex: wrapping years, 365-day forecast starting in
@@ -177,7 +179,8 @@ def perform_climatology_esp(
         # Setup the initial states from the warm-up and run the model.
         # Note that info on start/end dates and timeseries are in the kwds.
         m.resume(rvc)
-        m(run_name=f"run_{years}", **kwds)
+        kwds["run_name"] = f"{original_run_name}_{years}"
+        m(**kwds)
 
         # Add member to the ensemble and retag the dates to the real forecast dates
         # (or else we will get dates from the climate dataset that cover all years)
