@@ -1,10 +1,11 @@
 import datetime as dt
 
 from ravenpy.models import HMETS
-from ravenpy.utilities.testdata import get_local_testdata
 
 SALMON_coords = (-123.3659, 54.4848)  # (lon, lat)
 
+era5 = "era5/tas_pr_20180101-20180108.nc"
+
 
 params = (
     9.5019,
@@ -32,8 +33,8 @@ params = (
 
 
 class TestRavenERA5:
-    def test_simple(self):
-        ts = get_local_testdata("era5/tas_pr_20180101-20180108.nc")
+    def test_simple(self, get_file):
+        ts = get_file(era5)
         model = HMETS()
         model(
             ts=ts,
@@ -46,6 +47,6 @@ class TestRavenERA5:
             latitude=54.4848,
             longitude=-123.3659,
             rain_snow_fraction="RAINSNOW_DINGMAN",
-            tas={"scale": 1.0, "offset": -273.15, "time_shift": -0.25},
-            pr={"scale": 24000.0, "offset": 0.0, "time_shift": -0.25},
+            tas={"time_shift": -0.25},
+            pr={"time_shift": -0.25},
         )
