@@ -1,17 +1,20 @@
 import datetime as dt
 import os
 import zipfile
-from dataclasses import astuple, replace
+from dataclasses import astuple
 from pathlib import Path
 
 import numpy as np
 import pytest
 
+from ravenpy.config import options
 from ravenpy.config.commands import (  # GriddedForcingCommand,; LandUseClassesCommand,; ObservationDataCommand,; SoilClassesCommand,; SoilProfilesCommand,; VegetationClassesCommand,
     ChannelProfileCommand,
+    CustomOutput,
     EvaluationPeriod,
     GridWeightsCommand,
     HRUStateVariableTableCommand,
+    RedirectToFileCommand,
     SBGroupPropertyMultiplierCommand,
     Sub,
 )
@@ -29,27 +32,36 @@ from ravenpy.models import (
     RavenError,
     get_average_annual_runoff,
 )
-from ravenpy.utilities.testdata import get_local_testdata
 
-from .common import _convert_2d
-
-TS = get_local_testdata(
-    "raven-gr4j-cemaneige/Salmon-River-Near-Prince-George_meteo_daily.nc"
-)
+from .common import _convert_2d, _convert_3d
 
 # Link to THREDDS Data Server netCDF testdata
-TDS = "https://pavics.ouranos.ca/twitcher/ows/proxy/thredds/dodsC/birdhouse/disk2/testdata/raven"
+TDS = "https://pavics.ouranos.ca/twitcher/ows/proxy/thredds/dodsC/birdhouse/testdata/raven"
 
 
 @pytest.fixture
-def input2d(tmpdir):
+def input2d(tmpdir, get_file):
     """Convert 1D input to 2D output by copying all the time series along a new region dimension."""
-    ds = _convert_2d(TS)
+    ts = get_file(salmon_river)
+
+    ds = _convert_2d(ts)
     fn_out = os.path.join(tmpdir, "input2d.nc")
     ds.to_netcdf(fn_out)
     return Path(fn_out)
 
 
+@pytest.fixture
+def input3d(tmpdir, get_file):
+    """Convert 1D input to 2D output by copying all the time series along a new region dimension."""
+    ts = get_file(salmon_river)
+
+    ds = _convert_3d(ts)
+    ds = ds.drop_vars("qobs")
+    fn_out = os.path.join(tmpdir, "input3d.nc")
+    ds.to_netcdf(fn_out)
+    return Path(fn_out)
+
+
 def test_race():
     model1 = GR4JCN()
     model1.config.rvi.suppress_output = True
@@ -75,28 +87,39 @@ salmon_land_hru_2 = dict(
     area=2000.0, elevation=835.0, latitude=54.123, longitude=-123.4234
 )
 
+salmon_river = "raven-gr4j-cemaneige/Salmon-River-Near-Prince-George_meteo_daily.nc"
+salmon_river_2d = (
+    "raven-gr4j-cemaneige/Salmon-River-Near-Prince-George_meteo_daily_2d.nc"
+)
+
 
 class TestGR4JCN:
-    def test_error(self):
-        model = GR4JCN()
+    def test_error(self, tmp_path, get_file):
+        ts = get_file(salmon_river)
+
+        model = GR4JCN(workdir=tmp_path)
 
         model.config.rvp.params = model.Params(
             0.529, -3.396, 407.29, 1.072, 16.9, 0.947
         )
 
         with pytest.raises(RavenError) as exc:
-            model(TS)
+            model(ts)
 
         assert "CHydroUnit constructor:: HRU 1 has a negative or zero area" in str(
             exc.value
         )
 
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
-        print(model.workdir)
         model.config.rvi.start_date = dt.datetime(2000, 1, 1)
         model.config.rvi.end_date = dt.datetime(2002, 1, 1)
         model.config.rvi.run_name = "test"
+        model.config.rvi.custom_output = CustomOutput(
+            "YEARLY", "AVERAGE", "PRECIP", "ENTIRE_WATERSHED"
+        )
 
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
@@ -106,7 +129,7 @@ class TestGR4JCN:
 
         total_area_in_m2 = model.config.rvh.hrus[0].area * 1000 * 1000
         model.config.rvp.avg_annual_runoff = get_average_annual_runoff(
-            TS, total_area_in_m2
+            ts, total_area_in_m2
         )
 
         np.testing.assert_almost_equal(
@@ -115,7 +138,7 @@ class TestGR4JCN:
 
         assert model.config.rvi.suppress_output == ""
 
-        model(TS)
+        model(ts)
 
         # ------------
         # Check quality (diagnostic) of simulated streamflow values
@@ -159,7 +182,7 @@ class TestGR4JCN:
         # Check parser
         # ------------
 
-        assert model.config.rvi.calendar == RVI.CalendarOptions.GREGORIAN.value
+        assert model.config.rvi.calendar == options.Calendar.GREGORIAN.value
 
         # ------------
         # Check saved HRU states saved in RVC
@@ -171,13 +194,16 @@ class TestGR4JCN:
         # ------------
         assert model.hydrograph.attrs["model_id"] == "gr4jcn"
 
-    def test_routing(self):
+        assert (
+            model.output_path / f"{model.config.rvi.run_name}-"
+            f"{model.psim}_PRECIP_Yearly_Average_ByWatershed.nc"
+        ).exists()
+
+    def test_routing(self, get_file):
         """We need at least 2 subbasins to activate routing."""
         model = GR4JCN()
 
-        ts_2d = get_local_testdata(
-            "raven-gr4j-cemaneige/Salmon-River-Near-Prince-George_meteo_daily_2d.nc"
-        )
+        ts_2d = get_file(salmon_river_2d)
 
         #########
         # R V I #
@@ -187,6 +213,7 @@ class TestGR4JCN:
         model.config.rvi.end_date = dt.datetime(2002, 1, 1)
         model.config.rvi.run_name = "test_gr4jcn_routing"
         model.config.rvi.routing = "ROUTE_DIFFUSIVE_WAVE"
+        model.config.rvi.write_forcing_functions = True
 
         #########
         # R V H #
@@ -377,6 +404,43 @@ class TestGR4JCN:
         d = model.diagnostics
         np.testing.assert_almost_equal(d["DIAG_NASH_SUTCLIFFE"], -0.0141168, 4)
 
+        assert len(list(model.output_path.glob("*ForcingFunctions.nc"))) == 1
+
+    def test_redirect_to_file(self, tmpdir, input2d, get_file):
+        ts = get_file(salmon_river)
+
+        model = GR4JCN()
+        model.config.rvi.start_date = dt.datetime(2000, 1, 1)
+        model.config.rvi.end_date = dt.datetime(2002, 1, 1)
+        model.config.rvi.run_name = "test"
+
+        model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
+
+        model.config.rvp.params = model.Params(
+            0.529, -3.396, 407.29, 1.072, 16.9, 0.947
+        )
+
+        total_area_in_m2 = model.config.rvh.hrus[0].area * 1000 * 1000
+        model.config.rvp.avg_annual_runoff = get_average_annual_runoff(
+            ts, total_area_in_m2
+        )
+
+        np.testing.assert_almost_equal(
+            model.config.rvp.avg_annual_runoff, 208.4805694844741
+        )
+
+        gw = GridWeightsCommand()
+        gw_path = tmpdir / Path("grid_weights.rvt")
+        gw_path.write_text(gw.to_rv() + "\n", "utf8")
+
+        rtf = RedirectToFileCommand(gw_path)
+        model.config.rvt.grid_weights = rtf
+
+        model(input2d)
+
+        # Should be 13.25446 to be identical to 1D case
+        np.testing.assert_almost_equal(model.q_sim.isel(time=-1).data, 12.51634, 5)
+
     def test_config_update(self):
         model = GR4JCN()
 
@@ -409,13 +473,15 @@ class TestGR4JCN:
         model.config.update("params", (0.529, -3.396, 407.29, 1.072, 16.9, 0.947))
         assert model.config.rvp.params.GR4J_X1 == 0.529
 
-    def test_run(self):
+    def test_run(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
 
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
         model(
-            TS,
+            ts,
             area=4250.6,
             elevation=843.0,
             latitude=54.4848,
@@ -429,13 +495,15 @@ class TestGR4JCN:
 
         np.testing.assert_almost_equal(d["DIAG_NASH_SUTCLIFFE"], -0.117301, 4)
 
-    def test_evaluation(self):
+    def test_evaluation(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
 
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
         model(
-            TS,
+            ts,
             area=4250.6,
             elevation=843.0,
             latitude=54.4848,
@@ -455,11 +523,13 @@ class TestGR4JCN:
         assert "DIAG_KLING_GUPTA" in d
         assert len(d["DIAG_RMSE"]) == 3  # ALL, period1, period2
 
-    def test_run_new_hrus_param(self):
+    def test_run_new_hrus_param(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             params=(0.529, -3.396, 407.29, 1.072, 16.9, 0.947),
@@ -471,13 +541,15 @@ class TestGR4JCN:
         np.testing.assert_almost_equal(d["DIAG_NASH_SUTCLIFFE"], -0.117301, 4)
 
     # @pytest.mark.skip
-    def test_overwrite(self):
+    def test_overwrite(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
 
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             params=(0.529, -3.396, 407.29, 1.072, 16.9, 0.947),
@@ -491,7 +563,7 @@ class TestGR4JCN:
         # Please remove when fixed!
         model.hydrograph.close()  # Needed with xarray 0.16.1
 
-        model(TS, params=(0.5289, -3.397, 407.3, 1.071, 16.89, 0.948), overwrite=True)
+        model(ts, params=(0.5289, -3.397, 407.3, 1.071, 16.89, 0.948), overwrite=True)
 
         qsim2 = model.q_sim.copy(deep=True)
         m2 = qsim2.mean()
@@ -508,18 +580,22 @@ class TestGR4JCN:
 
         np.testing.assert_almost_equal(d["DIAG_NASH_SUTCLIFFE"], -0.117315, 4)
 
-        model.config.rvc.hru_states[1] = HRUStateVariableTableCommand.Record(soil0=0)
+        model.config.rvc.hru_states[1] = HRUStateVariableTableCommand.Record(
+            index=1, data={"SOIL[0]": 0}
+        )
 
         # Set initial conditions explicitly
         model(
-            TS,
+            ts,
             end_date=dt.datetime(2001, 2, 1),
             # hru_state=HRUStateVariableTableCommand.Record(soil0=0),
             overwrite=True,
         )
         assert model.q_sim.isel(time=1).values[0] < qsim2.isel(time=1).values[0]
 
-    def test_resume(self):
+    def test_resume(self, get_file):
+        ts = get_file(salmon_river)
+
         model_ab = GR4JCN()
         model_ab.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
         kwargs = dict(
@@ -527,7 +603,7 @@ class TestGR4JCN:
         )
         # Reference run
         model_ab(
-            TS,
+            ts,
             run_name="run_ab",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2001, 1, 1),
@@ -538,7 +614,7 @@ class TestGR4JCN:
 
         model_a.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
         model_a(
-            TS,
+            ts,
             run_name="run_a",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2000, 7, 1),
@@ -552,7 +628,7 @@ class TestGR4JCN:
         model_a.resume()
 
         model_a(
-            TS,
+            ts,
             run_name="run_2",
             start_date=dt.datetime(2000, 7, 1),
             end_date=dt.datetime(2001, 1, 1),
@@ -571,7 +647,7 @@ class TestGR4JCN:
             rvc
         )  # <--------- And this is how you feed it to a brand new model.
         model_b(
-            TS,
+            ts,
             run_name="run_2",
             start_date=dt.datetime(2000, 7, 1),
             end_date=dt.datetime(2001, 1, 1),
@@ -588,15 +664,17 @@ class TestGR4JCN:
         # cumulative sums of precipitation over the run ?
         # assert model_b.solution == model_ab.solution # This does not work. Atmosphere attributes are off.
 
-    def test_resume_earlier(self):
+    def test_resume_earlier(self, get_file):
         """Check that we can resume a run with the start date set at another date than the time stamp in the
         solution."""
+        ts = get_file(salmon_river)
+
         params = (0.529, -3.396, 407.29, 1.072, 16.9, 0.947)
         # Reference run
         model = GR4JCN()
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
         model(
-            TS,
+            ts,
             run_name="run_a",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2000, 2, 1),
@@ -617,7 +695,7 @@ class TestGR4JCN:
         model.config.rvc.parse_solution(rvc.read_text())
 
         model(
-            TS,
+            ts,
             run_name="run_b",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2000, 2, 1),
@@ -627,13 +705,15 @@ class TestGR4JCN:
         s_b = model.storage["Soil Water[0]"].isel(time=-1)
         assert s_a != s_b
 
-    def test_update_soil_water(self):
+    def test_update_soil_water(self, get_file):
+        ts = get_file(salmon_river)
+
         params = (0.529, -3.396, 407.29, 1.072, 16.9, 0.947)
         # Reference run
         model = GR4JCN()
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
         model(
-            TS,
+            ts,
             run_name="run_a",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2000, 2, 1),
@@ -644,12 +724,10 @@ class TestGR4JCN:
         s_1 = float(model.storage["Soil Water[1]"].isel(time=-1).values)
 
         # hru_state = replace(model.rvc.hru_state, soil0=s_0, soil1=s_1)
-        model.config.rvc.hru_states[1] = replace(
-            model.config.rvc.hru_states[1], soil0=s_0, soil1=s_1
-        )
+        model.config.rvc.hru_states[1].data.update({"SOIL[0]": s_0, "SOIL[1]": s_1})
 
         model(
-            TS,
+            ts,
             run_name="run_b",
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2000, 2, 1),
@@ -667,12 +745,14 @@ class TestGR4JCN:
         model = GR4JCN()
         assert model.raven_version == "3.0.4"
 
-    def test_parallel_params(self):
+    def test_parallel_params(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN()
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             suppress_output=False,
@@ -712,6 +792,22 @@ class TestGR4JCN:
         z = zipfile.ZipFile(model.outputs["rv_config"])
         assert len(z.filelist) == 10
 
+    def test_multiple_gauge_stations(self, get_file):
+        ts = get_file("famine/famine_input.nc")
+
+        model = GR4JCN()
+        model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
+        model.config.rvi.start_date = dt.datetime(2000, 1, 1)
+        model.config.rvi.end_date = dt.datetime(2002, 1, 1)
+        model.config.update(params=(0.529, -3.396, 407.29, 1.072, 16.9, 0.947))
+        model.config.rvt.meteo_idx = [0, 1, 2]
+        model.config.rvt.hydro_idx = (1,)
+        model(
+            ts=[
+                ts,
+            ]
+        )
+
     @pytest.mark.online
     def test_dap(self):
         """Test Raven with DAP link instead of local netCDF file."""
@@ -735,16 +831,14 @@ class TestGR4JCN:
             "https://pavics.ouranos.ca/twitcher/ows/proxy/thredds/dodsC/birdhouse/ets"
             "/Watersheds_5797_cfcompliant.nc"
         )
+
         model = GR4JCN()
         config = dict(
             start_date=dt.datetime(2010, 6, 1),
             end_date=dt.datetime(2010, 6, 10),
-            nc_index=5600,
+            meteo_idx=5600,
             run_name="Test_run",
             rain_snow_fraction="RAINSNOW_DINGMAN",
-            tasmax={"offset": -273.15},
-            tasmin={"offset": -273.15},
-            pr={"scale": 86400.0},
             hrus=[
                 model.LandHRU(
                     area=3650.47, latitude=49.51, longitude=-95.72, elevation=330.59
@@ -754,9 +848,18 @@ class TestGR4JCN:
         )
         model(ts=CANOPEX_DAP, **config)
 
+        # Check unit transformation parameters are correctly inferred
+        pr = model.config.rvt._var_cmds["pr"]
+        assert pr.scale, pr.offset == (86400.0, 0.0)
+
+        tasmax = model.config.rvt._var_cmds["tasmax"]
+        assert tasmax.scale, tasmax.offset == (1, -273.15)
+
 
 class TestGR4JCN_OST:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = GR4JCN_OST()
         model.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
@@ -765,11 +868,13 @@ class TestGR4JCN_OST:
         high = (2.5, 10.0, 700.0, 7.0, 30.0, 1.0)
 
         model.configure(
-            get_local_testdata("ostrich-gr4j-cemaneige/OstRandomNumbers.txt")
+            get_file(
+                "ostrich-gr4j-cemaneige/OstRandomNumbers.txt",
+            )
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             lowerBounds=low,
@@ -818,7 +923,7 @@ class TestGR4JCN_OST:
         gr4j.config.rvh.hrus = (GR4JCN.LandHRU(**salmon_land_hru_1),)
 
         gr4j(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             params=model.calibrated_params,
@@ -830,7 +935,9 @@ class TestGR4JCN_OST:
 
 
 class TestHMETS:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = HMETS()
         params = (
             9.5019,
@@ -857,7 +964,7 @@ class TestHMETS:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             area=4250.6,
@@ -874,11 +981,15 @@ class TestHMETS:
 
 
 class TestHMETS_OST:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = HMETS_OST()
 
         model.configure(
-            get_local_testdata("ostrich-gr4j-cemaneige/OstRandomNumbers.txt")
+            get_file(
+                "ostrich-gr4j-cemaneige/OstRandomNumbers.txt",
+            )
         )
 
         # Parameter bounds
@@ -930,7 +1041,7 @@ class TestHMETS_OST:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
@@ -1020,7 +1131,7 @@ class TestHMETS_OST:
         #                                err_msg='calibrated NSE is not matching expected value')
         hmets = HMETS()
         hmets(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
@@ -1036,7 +1147,9 @@ class TestHMETS_OST:
 
 
 class TestMOHYSE:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = MOHYSE()
         params = (
             1.0,
@@ -1052,7 +1165,7 @@ class TestMOHYSE:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             area=4250.6,
@@ -1068,11 +1181,15 @@ class TestMOHYSE:
 
 
 class TestMOHYSE_OST:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = MOHYSE_OST()
 
         model.configure(
-            get_local_testdata("ostrich-gr4j-cemaneige/OstRandomNumbers.txt")
+            get_file(
+                "ostrich-gr4j-cemaneige/OstRandomNumbers.txt",
+            )
         )
 
         # Parameter bounds
@@ -1080,7 +1197,7 @@ class TestMOHYSE_OST:
         high_p = (20.0, 1.0, 20.0, 5.0, 0.5, 1.0, 1.0, 1.0, 15.0, 15.0)
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
@@ -1152,7 +1269,7 @@ class TestMOHYSE_OST:
         #                                err_msg='calibrated NSE is not matching expected value')
         mohyse = MOHYSE()
         mohyse(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
@@ -1168,7 +1285,9 @@ class TestMOHYSE_OST:
 
 
 class TestHBVEC:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = HBVEC()
         params = (
             0.05984519,
@@ -1195,7 +1314,7 @@ class TestHBVEC:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             area=4250.6,
@@ -1209,7 +1328,9 @@ class TestHBVEC:
         d = model.diagnostics
         np.testing.assert_almost_equal(d["DIAG_NASH_SUTCLIFFE"], 0.0186633, 4)
 
-    def test_evap(self):
+    def test_evap(self, get_file):
+        ts = get_file(salmon_river)
+
         model = HBVEC()
         params = (
             0.05984519,
@@ -1236,7 +1357,7 @@ class TestHBVEC:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(2000, 1, 1),
             end_date=dt.datetime(2002, 1, 1),
             area=4250.6,
@@ -1251,11 +1372,15 @@ class TestHBVEC:
 
 
 class TestHBVEC_OST:
-    def test_simple(self):
+    def test_simple(self, get_file):
+        ts = get_file(salmon_river)
+
         model = HBVEC_OST()
 
         model.configure(
-            get_local_testdata("ostrich-gr4j-cemaneige/OstRandomNumbers.txt")
+            get_file(
+                "ostrich-gr4j-cemaneige/OstRandomNumbers.txt",
+            )
         )
 
         # Parameter bounds
@@ -1307,7 +1432,7 @@ class TestHBVEC_OST:
         )
 
         model(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
@@ -1383,7 +1508,7 @@ class TestHBVEC_OST:
         #                                err_msg='calibrated NSE is not matching expected value')
         hbvec = HBVEC()
         hbvec(
-            TS,
+            ts,
             start_date=dt.datetime(1954, 1, 1),
             duration=208,
             area=4250.6,
