@@ -13,11 +13,10 @@ from ravenpy.extractors.routing_product import (
     RoutingProductShapefileExtractor,
 )
 from ravenpy.models import Raven, get_average_annual_runoff
-from ravenpy.utilities.testdata import get_local_testdata
 
 
 class TestRouting:
-    def test_lievre_tutorial(self):
+    def test_lievre_tutorial(self, get_file):
         """
         This test reproduces the Lievre tutorial setup:
 
@@ -29,20 +28,14 @@ class TestRouting:
         # Input files #
         ###############
 
-        routing_product_shp_path = get_local_testdata(
+        routing_product_shp_path = get_file(
             "raven-routing-sample/finalcat_hru_info.zip"
         )
 
-        vic_streaminputs_nc_path = get_local_testdata(
-            "raven-routing-sample/VIC_streaminputs.nc"
-        )
-        vic_temperatures_nc_path = get_local_testdata(
-            "raven-routing-sample/VIC_temperatures.nc"
-        )
+        vic_streaminputs_nc_path = get_file("raven-routing-sample/VIC_streaminputs.nc")
+        vic_temperatures_nc_path = get_file("raven-routing-sample/VIC_temperatures.nc")
 
-        observation_data_nc_path = get_local_testdata(
-            "raven-routing-sample/WSC02LE024.nc"
-        )
+        observation_data_nc_path = get_file("raven-routing-sample/WSC02LE024.nc")
 
         #########
         # Model #
@@ -173,6 +166,7 @@ class TestRouting:
             vic_temperatures_nc_path, routing_product_shp_path
         )
 
+        # Note that `set_nc_variables` does not infer scale and offset parameters.
         model.config.rvt.set_nc_variables(
             [
                 dict(
@@ -244,7 +238,7 @@ class TestRouting:
         ]:
             assert model.hydrograph.q_sim[d].item() == pytest.approx(q_sim)
 
-    def test_lievre_tutorial_v21(self):
+    def test_lievre_tutorial_v21(self, get_file):
         """
         This test reproduces the Lievre tutorial setup (with the Routing Product V2.1):
 
@@ -256,20 +250,12 @@ class TestRouting:
         # Input files #
         ###############
 
-        routing_product_shp_path = get_local_testdata(
-            "raven-routing-sample/lievre_hrus_v21.zip"
-        )
+        routing_product_shp_path = get_file("raven-routing-sample/lievre_hrus_v21.zip")
 
-        vic_streaminputs_nc_path = get_local_testdata(
-            "raven-routing-sample/VIC_streaminputs.nc"
-        )
-        vic_temperatures_nc_path = get_local_testdata(
-            "raven-routing-sample/VIC_temperatures.nc"
-        )
+        vic_streaminputs_nc_path = get_file("raven-routing-sample/VIC_streaminputs.nc")
+        vic_temperatures_nc_path = get_file("raven-routing-sample/VIC_temperatures.nc")
 
-        observation_data_nc_path = get_local_testdata(
-            "raven-routing-sample/WSC02LE024.nc"
-        )
+        observation_data_nc_path = get_file("raven-routing-sample/WSC02LE024.nc")
 
         #########
         # Model #
@@ -411,6 +397,7 @@ class TestRouting:
             vic_temperatures_nc_path, routing_product_shp_path
         )
 
+        # Note that `set_nc_variables` does not infer scale and offset parameters.
         model.config.rvt.set_nc_variables(
             [
                 dict(
